function rec_desc = recognize_object_instance(obj_desc, bot_desc, debug)
% Assumes instance is planar

if (~exist('debug','var'))
    debug = false;
end

% Initialize return value 
rec_desc.is_recognized = false;
rec_desc.rec_img_name  = '';
rec_desc.bbox          = [0 0 0 0];
rec_desc.rec_url       = '';


% Match SIFT features (DOG + MSER)
[smxo, smyo, smxb, smyb] = match_sift_features2(obj_desc.dog_sift,  bot_desc.dog_sift,  0.8);
[mmxo, mmyo, mmxb, mmyb] = match_sift_features2(obj_desc.mser_sift, bot_desc.mser_sift, 0.8);

% Concatentate both matches
mxo = [ smxo  mmxo ];    mxb = [ smxb  mmxb ];
myo = [ smyo  mmyo ];    myb = [ smyb  mmyb ];

% Show matches if debugging
if (debug)
  %  figure(1)
    show_matches(obj_desc.img, bot_desc.img, mxo, myo, mxb, myb);
end

% If less than four points found, abort
if (length(mxo) < 4)
    rec_desc.is_recognized = false;
    return;
end

% Fit a homography to the remaining points via RANSAC (Kovesi)
% Homography takes you from bot image to object image
try
    [H,inliers] = ransacfithomography([mxb;myb],[mxo;myo],0.01);
catch
    lasterr
    inliers = [];
    fprintf('Ransac error when comparing %s and %s\n',obj_desc.img_name, bot_desc.img_name);
end

% In a degenerate case, no inliers are returned
if (length(inliers)<4)
    rec_desc.is_recognized = false;
    return;
end

% Plot the book boundary in the botImg
r = size(obj_desc.img,1); c = size(obj_desc.img,2);

bPts = [ 0   c  c  0  0;
         0   0  r  r  0;
         1   1  1  1  1];

bPts = homoTrans(inv(H),bPts);

if (is_degenerate(bPts))
    rec_desc.is_recognized = false;
    return;
end

if (isempty(bot_desc.img))
    bot_desc.img = imread(bot_desc.img_name);
end

if (debug)
    figure(2);
    imshow(bot_desc.img);
    hold on;
    for i=1:4
        line([bPts(1,i); bPts(1,i+1)],[bPts(2,i); bPts(2,i+1)],'LineWidth',4);
    end
    hold off;
    
    drawnow;
end

% find region to transform
minRow = floor(max(min(bPts(2,:)),1));
maxRow = floor(min(max(bPts(2,:)),size(bot_desc.img,1)));
minCol = floor(max(min(bPts(1,:)),1));
maxCol = floor(min(max(bPts(1,:)),size(bot_desc.img,2)));

% Transform the image using this homography
%t_bot_img   = imTrans(bot_desc.img, H , [minRow maxRow minCol maxCol]);
% TEST CODE
tf = cp2tform([mxb(inliers)' myb(inliers)'],[mxo(inliers)' myo(inliers)'],'projective');
t_bot_img = imtransform(bot_desc.img,tf,'XData',[1 size(obj_desc.img,2)],'YData',[1 size(obj_desc.img,1)]);



t_bot_desc.dog_sift  = get_sift_features(t_bot_img, 'sift');
t_bot_desc.mser_sift = get_sift_features(t_bot_img, 'mser');

if (debug)
%    figure(3);
%    imshow(t_bot_img);
end

% Match SIFT features (DOG + MSER) (make SIFT stricter)
[smxo, smyo, smxb, smyb, snum_unq] = match_sift_features2(obj_desc.dog_sift,  t_bot_desc.dog_sift,  0.6);
[mmxo, mmyo, mmxb, mmyb, mnum_unq] = match_sift_features2(obj_desc.mser_sift, t_bot_desc.mser_sift, 0.8);

% Concatentate both matches
mxo = [ smxo  mmxo ];    mxb = [ smxb  mmxb ];
myo = [ smyo  mmyo ];    myb = [ smyb  mmyb ];

num_unq = snum_unq + mnum_unq;

% Show matches if debugging
if (debug)
    figure(3)
    show_matches(obj_desc.img, t_bot_img, mxo, myo, mxb, myb);
end

% Declare match if greater than 15 unique matches found
if (num_unq >= 15)
    rec_desc.is_recognized = true;
    rec_desc.rec_img_name  = bot_desc.img_name;
    rec_desc.obj_img_name  = obj_desc.img_name;
    rec_desc.bbox          = [minRow minCol maxRow-minRow maxCol-minCol];
    rec_desc.bpts          = bPts;
    rec_desc.rec_url       = obj_desc.img_url;
    rec_desc.num_matches   = length(mxo);
end




function val = is_degenerate(quadr)

% check if area is enough 
% From http://mathworld.wolfram.com/Quadrilateral.html
a = quadr(:,2) - quadr(:,1);
b = quadr(:,3) - quadr(:,2);
c = quadr(:,4) - quadr(:,3);
d = quadr(:,1) - quadr(:,4);

p = b+c;
q = a+b;

K = 0.5 * abs(p(1)*q(2) - p(2)*q(1));

% threshold (hard for now, set param later)
if (K < 30)
    val = true;
    return;
end

c1 = sign(a(1)*b(2) - a(2)*b(1));
c2 = sign(b(1)*c(2) - b(2)*c(1));
c3 = sign(c(1)*d(2) - c(2)*d(1));
c4 = sign(d(1)*a(2) - d(2)*a(1));

if (c1 > 0 && c2 > 0 && c3 > 0 && c4 > 0)
    val = false;
    return;
end

val = true;
return;






            